/*  smpcmpx1.cpp  by K.Tsuru  */
/*
This program compares SComplex class with C++ complex one.
It has been tested on gcc.
*/
#include "sn.h"
#include "wait.h"

static const char* const title = "A test program for SComplex class";

void ShowTitle(const char* func){
	printf("****************** test of ");
	printf("%s", func);
	puts(" ********************");
}
void NegativeLog(void){
	SComplex z(-1, 0.0), r;
	r = Clog(z);
	printf("log(-1.0) =");
	r.Put();
	fflush(stdout);
	Wait("\nimaginary part is pi?");
}
void TestCexp(void){
	ShowTitle("Cexp");
	SComplex z, q;
	SDouble theta;
	int i, N = 10;
	double dr, di, cf = -M_PI;
	complex <double> dz, dq;
	printf("theta      real(z)   imag(z)   real(sq)    imag(sq)  difference from double\n");
	for(i=-N; i<= N;i++){
		theta = 2*i/(double)N;
		z = SComplex(theta, cf*theta);
		dz = complex <double>(doubleD(theta), cf*doubleD(theta));
		dq = exp(dz);
		q = Cexp(z);
		dr = real(dq)-doubleD(q.Real());
		di = imag(dq)-doubleD(q.Imag());
		printf("%6.1f %10.6f %10.6f %10.6f %10.6f %10.5e %10.5e\n", doubleD(theta), doubleD(z.Real()), doubleD(z.Imag()), doubleD(q.Real()), doubleD(q.Imag()), fabs(dr), fabs(di));
	}
}
void TestClog(void){
	ShowTitle("Clog");
	SComplex z, q;
	SDouble theta, pi = Pi();
	int i, N = 10;
	double dr, di, cf = -M_PI;
	complex <double> dz, dq;
	printf("theta/pi   real(z)   imag(z)    real(q)     imag(q)  difference from double\n");
	for(i=-N; i<= N;i++){
		theta = 2*i/(double)N;
		z = SComplex(Cos(theta*pi), Sin(-theta*pi));
		z *= cf;
		dz = complex <double>(cos(doubleD(theta*pi)), sin(-doubleD(theta*pi)));
		dz *= cf;
		dq = log(dz);
		q = Clog(z);
		dr = real(dq)-doubleD(q.Real());
		di = imag(dq)-doubleD(q.Imag());
		printf("%6.1f %10.6f %10.6f %10.6f %10.6f %10.5e %10.5e\n", doubleD(theta), doubleD(z.Real()), doubleD(z.Imag()), doubleD(q.Real()), doubleD(q.Imag()), fabs(dr), fabs(di));
	}
}
void TestCsin(void){
	ShowTitle("Csin");
	SComplex z, q;
	SDouble theta, pi = Pi(), r;
	int i, N = 10;
	double dr, di;
	complex <double> dz, dq;
	printf("theta/pi   real(z)   imag(z)   real(sq)    imag(sq)  difference from double\n");
	for(i=-N; i<= N;i++){
		theta = i/(double)N;
		z = SComplex(theta*pi, theta*pi);
		dz = complex <double>(doubleD(theta*pi), doubleD(theta*pi));
		dq = sin(dz);
		q = Csin(z);
		dr = real(dq)-doubleD(q.Real());
		di = imag(dq)-doubleD(q.Imag());
		printf("%6.1f %10.6f %10.6f %10.6f %10.6f %10.5e %10.5e\n", doubleD(theta), doubleD(z.Real()), doubleD(z.Imag()), doubleD(q.Real()), doubleD(q.Imag()), fabs(dr), fabs(di));
	}
}
void TestCcos(void){
	ShowTitle("Ccos");
	SComplex z, q;
	SDouble theta, pi = Pi(), r;
	int i, N = 10;
	double dr, di;
	complex <double> dz, dq;
	printf("theta/pi   real(z)   imag(z)   real(sq)    imag(sq)  difference from double\n");
	for(i=-N; i<= N;i++){
		theta = i/(double)N;
		z = SComplex(theta*pi, theta*pi);
		dz = complex <double>(doubleD(theta*pi), doubleD(theta*pi));
		dq = cos(dz);
		q = Ccos(z);
		dr = real(dq)-doubleD(q.Real());
		di = imag(dq)-doubleD(q.Imag());
		printf("%6.1f %10.6f %10.6f %10.6f %10.6f %10.5e %10.5e\n", doubleD(theta), doubleD(z.Real()), doubleD(z.Imag()), doubleD(q.Real()), doubleD(q.Imag()), fabs(dr), fabs(di));
	}
}
void TestCsinh(void){
	ShowTitle("Csinh");
	SComplex z, q;
	SDouble theta, pi = Pi(), r;
	int i, N = 10;
	double dr, di;
	complex <double> dz, dq;
	printf("theta/pi   real(z)   imag(z)   real(sq)    imag(sq)  difference from double\n");
	for(i=-N; i<= N;i++){
		theta = i/(double)N;
		z = SComplex(theta*pi, theta*pi);
		dz = complex <double>(doubleD(theta*pi), doubleD(theta*pi));
		dq = sinh(dz);
		q = Csinh(z);
		dr = real(dq)-doubleD(q.Real());
		di = imag(dq)-doubleD(q.Imag());
		printf("%6.1f %10.6f %10.6f %10.6f %10.6f %10.5e %10.5e\n", doubleD(theta), doubleD(z.Real()), doubleD(z.Imag()), doubleD(q.Real()), doubleD(q.Imag()), fabs(dr), fabs(di));
	}
}
void TestCcosh(void){
	ShowTitle("Ccosh");
	SComplex z, q;
	SDouble theta, pi = Pi(), r;
	int i, N = 10;
	double dr, di;
	complex <double> dz, dq;
	printf("theta/pi   real(z)   imag(z)   real(sq)    imag(sq)  difference from double\n");
	for(i=-N; i<= N;i++){
		theta = i/(double)N;
		z = SComplex(theta*pi, theta*pi);
		dz = complex <double>(doubleD(theta*pi), doubleD(theta*pi));
		dq = cosh(dz);
		q = Ccosh(z);
		dr = real(dq)-doubleD(q.Real());
		di = imag(dq)-doubleD(q.Imag());
		printf("%6.1f %10.6f %10.6f %10.6f %10.6f %10.5e %10.5e\n", doubleD(theta), doubleD(z.Real()), doubleD(z.Imag()), doubleD(q.Real()), doubleD(q.Imag()), fabs(dr), fabs(di));
	}
}
void TestCsqrt(void){
	ShowTitle("Csqrt");
	SComplex z, q;
	SDouble theta, pi = Pi();
	int i, N = 10;
	double dr, di;
	complex <double> dz, dq;
	printf("theta/pi   real(z)   imag(z)   real(sq)    imag(sq)  difference from double\n");
	for(i=-N; i<= N;i++){
		theta = 2*i/(double)N;
		z = SComplex(Cos(theta*pi), Sin(theta*pi));
		dz = complex <double>(cos(doubleD(theta*pi)), sin(doubleD(theta*pi)));
		dq = sqrt(dz);
		q = Csqrt(z);
		dr =real(dq)-doubleD(q.Real());
		di =imag(dq)-doubleD(q.Imag());
		printf("%6.1f %10.6f %10.6f %10.6f %10.6f %10.5e %10.5e\n", doubleD(theta), doubleD(z.Real()), doubleD(z.Imag()), doubleD(q.Real()), doubleD(q.Imag()), fabs(dr), fabs(di));
	}
}
int main(){
	RealSize C;
	C.SetEffFig(5);

	ErrorPuts(title);
	NegativeLog();
	TestCcosh();
	Wait();
	TestCsinh();
	Wait();
	TestCcos();
	Wait();
	TestCsin();
	Wait();
	TestCexp();
	Wait();
	TestClog();
	Wait();
	TestCsqrt();
	Wait();
	return 0;
}
